import "Turbine";
import "Turbine.Gameplay";
import "Gillothrin.Items.Wallet";
import "Gillothrin.Items.ItemsWindow";
import "Gillothrin.Items.DataOptions";
import "Gillothrin.Items.OptionsControl";

ItemsPlugin = class(Turbine.Object);
function ItemsPlugin:Constructor()
  Turbine.Object.Constructor(self);

  self.optionsObj = DataOptions();
  self.itemRemovedPlaceHolder = {};

  local player = Turbine.Gameplay.LocalPlayer:GetInstance();
  self.playerWallet=Wallet(player,self.optionsObj);

  self.itemsWindow = ItemsWindow(self.optionsObj,self.playerWallet);
  self.optionsControl = OptionsControl(self.optionsObj,self.playerWallet);

  AddCallback(player:GetWallet(), "ItemAdded", eventWalletChange);
  AddCallback(player:GetWallet(), "ItemRemoved", eventWalletItemRemoved);  
  
  self.itemsWindow:LoadWindow();
end

_G.eventWalletChange = function(sender, args)
  itemsPlugin.itemsWindow:RepaintItemInfo();
  itemsPlugin.optionsControl:RepaintOptionsControl();
end

--[[

	Ugly hack because the Wallet.ItemRemoved event is triggering before the item is actually 

	removed from the wallet making a refresh then ineffective. It should work like the item 

	triggers an QuantityChange event and THEN the wallet triggers an ItemRemoved event 

	(when the item is gone from the wallet).

]]--  
_G.eventWalletItemRemoved = function(s,a)

  local obj = Turbine.Object();

  if a.Item:GetQuantity() > 1 then
    obj.items = a.Item:GetQuantity();
  end
  obj.name = a.Item:GetName();
  table.insert(itemsPlugin.itemRemovedPlaceHolder,obj);

  AddCallback(Turbine.Chat,"Received",chatItemRemovedEvent);

end

_G.chatItemRemovedEvent = function(s,a)
  if a.ChatType == Turbine.ChatType.SelfLoot then
    local pkeys = {};
    for k,v in pairs(itemsPlugin.itemRemovedPlaceHolder) do
      table.insert(pkeys,k);
    end
    local m = false;
    for _,k in ipairs(pkeys) do
      local rmsg = L["ItemRemoved"];
      if a.Message:sub(0,string.len(rmsg)) == rmsg then
        local name = GetMessageData(a.Message);
        if name == itemsPlugin.itemRemovedPlaceHolder[k].name then
          itemsPlugin.itemRemovedPlaceHolder[k] = nil;
          break;
        end
      end
    end
    local cnt = 0;
    for k,v in pairs(itemsPlugin.itemRemovedPlaceHolder) do cnt = cnt + 1; end
    if cnt == 0 then
      itemsPlugin.itemsWindow:RepaintItemInfo();
      RemoveCallback(Turbine.Chat,"Received",chatItemRemovedEvent);
    end
  end
end

function ItemsPlugin:Unload()
  local player = Turbine.Gameplay.LocalPlayer:GetInstance();
  RemoveCallback(player:GetWallet(), "ItemAdded", eventWalletChange);
  RemoveCallback(player:GetWallet(), "ItemRemoved", eventWalletItemRemoved);
  RemoveCallback(Turbine.Chat,"Received",chatItemRemovedEvent);

  _G.eventWalletChange = nil;
  _G.eventWalletItemRemoved = nil;
  _G.chatItemRemovedEvent = nil;

  if Turbine.Shell.IsCommand(shellCommandName) then
    Turbine.Shell.RemoveCommand(shellCommandName);
  end

  self.optionsObj:SaveData();
  self.optionsControl:Unload();
  self.itemsWindow:Unload();

end

function ItemsPlugin:GetOptionsPanel()
  self.optionsControl:RepaintOptionsControl();
  return self.optionsControl;
end

function ItemsPlugin:RegisterShellCommand()
  myCommandLine = Turbine.ShellCommand();
  local this = self;
  function myCommandLine:Execute( name, shellCommand )
    shellCommand = string.lower(shellCommand);
    if (shellCommand == "setup") or (shellCommand == "options") then
      this.itemsWindow:SetVisible();
      this.optionsControl:ShowOptionsControl();
    elseif (shellCommand == "show") or (shellCommand == "on") then
      this.itemsWindow:SetVisible(true);
      this.itemsWindow:Activate();
    elseif (shellCommand == "hide") or (shellCommand == "off") then
      this.itemsWindow:Close();
    elseif shellCommand == "refresh" then
      eventWalletChange();
    elseif shellCommand == "reset" then
      this.itemsWindow:ResetWindow();
    else
      myCommandLine:GetHelp();
    end
  end

  function myCommandLine:GetShortHelp()
    echo("Command: <rgb=#FFFFFF>/"..shellCommandName.."</rgb> [show|on] [hide|off] [setup|options] [refresh] [reset] [help]");
  end

  function myCommandLine:GetHelp()
    echo(string.rep("=",string.len(pluginFullIdText)));
    echo(pluginFullIdText);
    echo(string.rep("=",string.len(pluginFullIdText)));
    echo("USAGE: /"..shellCommandName.." [command]\n");
    echo("COMMANDS:");
    echo("help - show this help (also if no command)");
    echo("show|on - show item tracking window");
    echo("hide|off - hide item tracking window");
    echo("setup|options - show the options window");
    echo("refresh - refresh your tracked items");
    echo("reset - reset the window to default");
    echo("");
  end
  return myCommandLine;
end
