import "Gillothrin.Items.VindarPatch";

Wallet = class(Turbine.Object);
function Wallet:Constructor(player,optionsObj)
  Turbine.Object.Constructor( self );
  self.optionsObj = optionsObj;
  self.playerWallet = player:GetWallet();
  self.items = {};
  self.moneyUpdater = MoneyUpdater(player:GetAttributes(),optionsObj);
end

function Wallet:GetFoundItems()
  return self.items;
end

function Wallet:GetLOTROWallet()
  return self.playerWallet;
end

function Wallet:LoadWallet()
  self.size = self.playerWallet:GetSize();
  local nextSet = {};
  for f=1, self.size do
    local walletItem = self.playerWallet:GetItem(f);
    local name = walletItem:GetName();
    local ico = walletItem:GetSmallImage();
    self.optionsObj.seenWalletItems[name] = ico;
    nextSet[name]= VWalletItem(name, ico, walletItem);
  end

  self.moneyUpdater:LoadGold(nextSet);

  for k,v in pairs(self.optionsObj.seenWalletItems) do
    if nextSet[k] == nil then
      nextSet[k] = VWalletItem(k,v,nil);
    end
  end
  for k,v in pairs(self.items) do
    v:Unhook();
  end
  self.items = nextSet;
end

function Wallet:Unload()
  self.moneyUpdater:Unload();
end

VWalletItem = class(Turbine.Object);
function VWalletItem:Constructor(name,ico,walletItem,quantity)
  Turbine.Object.Constructor(self);
  self.name = name;
  self.ico = ico;
  self.walletItem = walletItem;
  self.quantity = quantity;
end

function VWalletItem:GetName()
  return self.name;
end

function VWalletItem:GetQuantity()
  if(self.walletItem ~= nil) then
    return self.walletItem:GetQuantity();
  end
  if self.quantity ~= nil then
    return self.quantity;
  end
  return 0;
end

function VWalletItem:GetSmallImage()
  return self.ico;
end

function VWalletItem:Listen(f)
  if self.walletItem ~= nil then
    self.listener = f;
    AddCallback(self.walletItem,"QuantityChanged",f);
  end
end

function VWalletItem:Unhook()
  if self.walletItem ~= nil then
    RemoveCallback(self.walletItem,"QuantityChanged",self.listener);
  end
end

MoneyUpdater = class (Turbine.UI.Control);
function MoneyUpdater:Constructor(attributes,optionsObj)
  Turbine.UI.Control.Constructor(self);
  self.gold = L["gold"];
  self.silver = L["silver"];
  self.copper = L["copper"];
  self.playerAttributes = attributes;
  optionsObj.seenWalletItems[self.gold] = 0x41007e7b;
  optionsObj.seenWalletItems[self.silver] = 0x41007e7c;
  optionsObj.seenWalletItems[self.copper] = 0x41007e7d;
  self.ticks = 0;
  self.UpdateHandler = function(s,a)
    if(self.ticks == 100) then
      self.ticks = 0;
      local g,s,c = self:GetGold();
      self:CheckMoney(self.goldItem,g);
      self:CheckMoney(self.silverItem,s);
      self:CheckMoney(self.copperItem,c);
    else
      self.ticks = self.ticks + 1;
    end
  end
  AddCallback(self,"Update",self.UpdateHandler);
end

function MoneyUpdater:CheckForUpdates()
  f = function (item)
    if item ~= nil then
      return item.UpdateHandler == nil;
    end
    return true;
  end
  if f(self.goldItem) and f(self.silverItem) and f(self.copperItem) then
    self:SetWantsUpdates(false);
  end
end

function MoneyUpdater:ToggleUpdates()
   if not self:GetWantsUpdates() then
    self:SetWantsUpdates(true);
  end
end

function MoneyUpdater:CheckMoney(item,v)
  if item:GetQuantity() ~= v then
    item:SetQuantity(v);
    if item.QuantityChanged ~= nil then
      item:QuantityChanged(self,v);
    end
  end
end

function MoneyUpdater:GetGold()
  local total = self.playerAttributes:GetMoney();
  local g,s,c = 0,0,math.fmod(total,100);
  total = (total - c)/100;
  s = math.fmod(total,1000);
  g = (total - s)/1000;
  return g,s,c;
end

function MoneyUpdater:LoadGold(nextSet)
  local g,s,c = self:GetGold();

  self.goldItem = MoneyItem(g);
  self.silverItem = MoneyItem(s);
  self.copperItem = MoneyItem(c);

  nextSet[self.gold] = MoneyWalletItem(self.gold,0x41007e7b,self.goldItem,g,self);
  nextSet[self.silver] = MoneyWalletItem(self.silver,0x41007e7c,self.silverItem,s,self);
  nextSet[self.copper] = MoneyWalletItem(self.copper,0x41007e7d,self.copperItem,c,self);
  
end

function MoneyUpdater:Unload()
  self:SetWantsUpdates(false);
  RemoveCallback(self,"Update",self.UpdateHandler);
end

MoneyWalletItem = class(Turbine.Object)
function MoneyWalletItem:Constructor(name,ico,walletItem,quantity,control)
  Turbine.Object.Constructor(self);
  self.name = name;
  self.ico = ico;
  self.walletItem = walletItem;
  self.quantity = quantity;
  self.control = control;
end

function MoneyWalletItem:GetName()
  return self.name;
end

function MoneyWalletItem:GetQuantity()
  return self.walletItem:GetQuantity();
end

function MoneyWalletItem:GetSmallImage()
  return self.ico;
end

function MoneyWalletItem:Listen(f)
  self.listener = f;
  AddCallback(self.walletItem,"QuantityChanged",f);
  self.control:ToggleUpdates();
end

function MoneyWalletItem:Unhook()
  self.control:CheckForUpdates();
  RemoveCallback(self.walletItem,"QuantityChanged",self.listener);
end

MoneyItem = class (Turbine.Object)
function MoneyItem:Constructor(quantity)
  Turbine.Object.Constructor(self);
  self.quantity = quantity;
end

function MoneyItem:SetQuantity(q)
  self.quantity = q;
end

function MoneyItem:GetQuantity()
  return self.quantity;
end
